/*
    irceditaccountwidget.cpp - IRC Account Widget

    Copyright (c) 2003      by Olivier Goffart  <ogoffart@kde.org>
    Copyright (c) 2003      by Jason Keirstead  <jason@keirstead.org>
    Copyright (c) 2003-2007 by Michel Hermier <michel.hermier@gmail.com>

    Kopete    (c) 2003 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "irceditaccountwidget.moc"

#include "ircaccount.h"
#include "irccontact.h"
#include "ircnetwork.h"
#include "ircprotocol.h"

#include "kopetepasswordwidget.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kcharsets.h>
#include <kopetepassword.h>

#include <qlabel.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qvalidator.h>
#include <qcombobox.h>

#include <qlineedit.h>
#include <qtextcodec.h>

#include <algorithm>
#include <kconfiggroup.h>

IRCEditAccountWidget::IRCEditAccountWidget(IRCAccount *ident, QWidget *parent)
	: QWidget(parent), KopeteEditAccountWidget(ident)
{

	setupUi(this);


//	qDebug() << "editaccount widget constructor";

	QString encoding = "UTF-8";

	if( account() )
	{

		nickName->setText(  account()->nick );
		nickName2->setText( account()->nick2);
		realName->setText( account()->realname );
		password->load(&account()->password());


		preferSSL->setChecked(account()->preferssl);
		autoConnect->setChecked( account()->excludeConnect() );


		foreach (QString item,account()->connectCommands)
			commandList->addItem(item);

		encoding = account()->encoding;

	}

//	qDebug() << "loaded settings";

	KCharsets *c = KGlobal::charsets();
	charset->addItems( c->availableEncodingNames() );



	for( int i = 0; i < charset->count(); ++i )
	{
		if( charset->itemText(i) == encoding )
		{
			charset->setCurrentIndex( i );
			break;
		}
	}


	connect( addButton, SIGNAL( clicked() ), this, SLOT( slotAddCommand() ) );
	connect( removeButton, SIGNAL( clicked() ), this, SLOT( slotRemoveCommand() ) );

	connect( editButton, SIGNAL( clicked() ), this, SLOT(slotEditNetworks() ) );

//	connect( network, SIGNAL( activated( const QString & ) ),
//	this, SLOT( slotUpdateNetworkDescription( const QString &) ) );

	slotUpdateNetworks( QString() );
}

IRCEditAccountWidget::~IRCEditAccountWidget()
{
}

IRCAccount *IRCEditAccountWidget::account ()
{
	return dynamic_cast<IRCAccount *>(KopeteEditAccountWidget::account () );
}

struct NetNameComparator {

	bool operator()(const IRC::Network& a, const IRC::Network& b) const
	{
		return a.name < b.name;
	}
};

void IRCEditAccountWidget::slotUpdateNetworks( const QString & selectedNetwork )
{

	network->clear();
	kDebug( 14210 ) << "updating networks. selected=" << selectedNetwork;

	IRC::NetworkList networks = IRC::Networks::self()->networks();
	std::sort(networks.begin(), networks.end(), NetNameComparator());

	kDebug()<<"got "<<networks.size()<<" networks"<<endl;
	uint i = 0;
	foreach(const IRC::Network& net, networks) {
		network->addItem(net.name);

		if ((account() && account()->network == net.name)
				|| net.name == selectedNetwork)
		{
			network->setCurrentIndex( i );
		}
		++i;
	}
}

void IRCEditAccountWidget::slotEditNetworks()
{
	IRCProtocol::protocol()->editNetworks(network->currentText(), this);
	qDebug() << "do slotupdate networks" ;
	slotUpdateNetworks( network->currentText() );

}



void IRCEditAccountWidget::slotRemoveCommand()
{
	if (commandList->currentIndex().row()>=0)
	{
		delete commandList->takeItem(commandList->currentIndex().row());
	}
}


void IRCEditAccountWidget::slotAddCommand()
{

	if ( !commandEdit->text().isEmpty() )
	{
		commandList->addItem( commandEdit->text());
		commandEdit->clear();
	}

}


QString IRCEditAccountWidget::generateAccountId( const QString &network )
{
	KSharedConfig::Ptr config = KGlobal::config();
	QString nextId = network;

	uint accountNumber = 1;
	while( config->hasGroup( QString("Account_%1_%2").arg( IRCProtocol::protocol()->pluginId() ).arg( nextId ) ) )
	{
		nextId = QString::fromLatin1("%1_%2").arg( network ).arg( ++accountNumber );
	}
	kDebug( 14120 ) << " ID IS: " << nextId;
	return nextId;
}

Kopete::Account *IRCEditAccountWidget::apply()
{

	IRCAccount *a = account();

	if( !a )
	{
		kDebug( 14210 ) << "have no account, create new account ";
		a = new IRCAccount(IRCProtocol::protocol(), generateAccountId(network->currentText()) );
		setAccount( a );
	}

	a->network = network->currentText();
	a->nick = nickName->text();
	a->nick2 = nickName2->text();
	a->realname =  realName->text();
	a->preferssl = preferSSL->isChecked();

	a->connectCommands.clear();
	for (int i=0;i<commandList->count();i++)
		a->connectCommands.append(commandList->item(i)->text());


	password->save(&a->password());

	a->encoding = charset->currentText();

	a->storeSettings();


	return account();
}

bool IRCEditAccountWidget::validateData()
{
	if( nickName->text().isEmpty() )
		KMessageBox::sorry(this, i18n("<qt>You must enter a nickname.</qt>"), i18n("Kopete"));
	else
		return true;

	return false;
}

